@@ -26,8 +26,8 @@ from hypothesis import (
     strategies as st,
 )
 from hypothesis.database import ExampleDatabase, InMemoryExampleDatabase
-from hypothesis.errors import FailedHealthCheck, Flaky
-from hypothesis.internal.compat import bit_count, int_from_bytes
+from hypothesis.errors import FailedHealthCheck, FlakyStrategyDefinition
+from hypothesis.internal.compat import PYPY, bit_count, int_from_bytes
 from hypothesis.internal.conjecture import engine as engine_module
 from hypothesis.internal.conjecture.data import ConjectureData, IRNode, Overrun, Status
 from hypothesis.internal.conjecture.datatree import compute_max_children
@@ -58,7 +58,7 @@ from tests.conjecture.common import (
 def test_can_index_results():
     @run_to_buffer
     def f(data):
-        data.draw_bytes(5)
+        data.draw_bytes(5, 5)
         data.mark_interesting()
 
     assert f.index(0) == 0
@@ -68,8 +68,8 @@ def test_can_index_results():
 def test_non_cloneable_intervals():
     @run_to_buffer
     def x(data):
-        data.draw_bytes(10)
-        data.draw_bytes(9)
+        data.draw_bytes(10, 10)
+        data.draw_bytes(9, 9)
         data.mark_interesting()
 
     assert x == bytes(19)
@@ -79,7 +79,7 @@ def test_deletable_draws():
     @run_to_buffer
     def x(data):
         while True:
-            x = data.draw_bytes(2)
+            x = data.draw_bytes(2, 2)
             if x[0] == 255:
                 data.mark_interesting()
 
@@ -97,7 +97,7 @@ def test_can_load_data_from_a_corpus():
     db.save(key, value)
 
     def f(data):
-        if data.draw_bytes(len(value)) == value:
+        if data.draw_bytes(len(value), len(value)) == value:
             data.mark_interesting()
 
     runner = ConjectureRunner(f, settings=settings(database=db), database_key=key)
@@ -148,7 +148,7 @@ def test_detects_flakiness():
     count = [0]
 
     def tf(data):
-        data.draw_bytes(1)
+        data.draw_bytes(1, 1)
         count[0] += 1
         if not failed_once[0]:
             failed_once[0] = True
@@ -165,6 +165,7 @@ def recur(i, data):
         recur(i - 1, data)
 
 
+@pytest.mark.skipif(PYPY, reason="stack tricks only work reliably on CPython")
 def test_recursion_error_is_not_flaky():
     def tf(data):
         i = data.draw_integer(0, 2**16 - 1)
@@ -173,7 +174,7 @@ def test_recursion_error_is_not_flaky():
         except RecursionError:
             data.mark_interesting()
 
-    runner = ConjectureRunner(tf)
+    runner = ConjectureRunner(tf, settings=settings(derandomize=True))
     runner.run()
     assert runner.exit_reason == ExitReason.finished
 
@@ -183,9 +184,9 @@ def test_variadic_draw():
         result = []
         while True:
             data.start_example(SOME_LABEL)
-            d = data.draw_bytes(1)[0] & 7
+            d = data.draw_bytes(1, 1)[0] & 7
             if d:
-                result.append(data.draw_bytes(d))
+                result.append(data.draw_bytes(d, d))
             data.stop_example()
             if not d:
                 break
@@ -204,8 +205,8 @@ def test_variadic_draw():
 def test_draw_to_overrun():
     @run_to_buffer
     def x(data):
-        d = (data.draw_bytes(1)[0] - 8) & 0xFF
-        data.draw_bytes(128 * d)
+        d = (data.draw_bytes(1, 1)[0] - 8) & 0xFF
+        data.draw_bytes(128 * d, 128 * d)
         if d >= 2:
             data.mark_interesting()
 
@@ -214,8 +215,8 @@ def test_draw_to_overrun():
 
 def test_can_navigate_to_a_valid_example():
     def f(data):
-        i = int_from_bytes(data.draw_bytes(2))
-        data.draw_bytes(i)
+        i = int_from_bytes(data.draw_bytes(2, 2))
+        data.draw_bytes(i, i)
         data.mark_interesting()
 
     runner = ConjectureRunner(f, settings=settings(max_examples=5000, database=None))
@@ -234,7 +235,7 @@ def test_stops_after_max_examples_when_reading():
     seen = []
 
     def f(data):
-        seen.append(data.draw_bytes(1))
+        seen.append(data.draw_bytes(1, 1))
 
     runner = ConjectureRunner(
         f, settings=settings(max_examples=1, database=db), database_key=key
@@ -247,7 +248,7 @@ def test_stops_after_max_examples_when_generating():
     seen = []
 
     def f(data):
-        seen.append(data.draw_bytes(1))
+        seen.append(data.draw_bytes(1, 1))
 
     runner = ConjectureRunner(f, settings=settings(max_examples=1, database=None))
     runner.run()
@@ -285,10 +286,10 @@ def test_interleaving_engines():
 
     @run_to_buffer
     def x(data):
-        rnd = Random(data.draw_bytes(1))
+        rnd = Random(data.draw_bytes(1, 1))
 
         def g(d2):
-            d2.draw_bytes(1)
+            d2.draw_bytes(1, 1)
             data.mark_interesting()
 
         runner = ConjectureRunner(g, random=rnd)
@@ -306,7 +307,7 @@ def test_phases_can_disable_shrinking():
     seen = set()
 
     def f(data):
-        seen.add(bytes(data.draw_bytes(32)))
+        seen.add(bytes(data.draw_bytes(32, 32)))
         data.mark_interesting()
 
     runner = ConjectureRunner(
@@ -338,12 +339,12 @@ def test_reuse_phase_runs_for_max_examples_if_generation_is_disabled():
 def test_erratic_draws():
     n = [0]
 
-    with pytest.raises(Flaky):
+    with pytest.raises(FlakyStrategyDefinition):
 
         @run_to_buffer
         def x(data):
-            data.draw_bytes(n[0])
-            data.draw_bytes(255 - n[0])
+            data.draw_bytes(n[0], n[0])
+            data.draw_bytes(255 - n[0], 255 - n[0])
             if n[0] == 255:
                 data.mark_interesting()
             else:
@@ -368,10 +369,10 @@ def test_one_dead_branch():
 
         @run_to_buffer
         def x(data):
-            i = data.draw_bytes(1)[0]
+            i = data.draw_bytes(1, 1)[0]
             if i > 0:
                 data.mark_invalid()
-            i = data.draw_bytes(1)[0]
+            i = data.draw_bytes(1, 1)[0]
             if len(seen) < 255:
                 seen.add(i)
             elif i not in seen:
@@ -398,7 +399,7 @@ def test_returns_written():
 
     @run_to_buffer
     def written(data):
-        data.draw_bytes(len(value), forced=value)
+        data.draw_bytes(len(value), len(value), forced=value)
         data.mark_interesting()
 
     assert value == written
@@ -424,21 +425,21 @@ def fails_health_check(label, **kwargs):
 def test_fails_health_check_for_all_invalid():
     @fails_health_check(HealthCheck.filter_too_much)
     def _(data):
-        data.draw_bytes(2)
+        data.draw_bytes(2, 2)
         data.mark_invalid()
 
 
 def test_fails_health_check_for_large_base():
     @fails_health_check(HealthCheck.large_base_example)
     def _(data):
-        data.draw_bytes(10**6)
+        data.draw_bytes(10**6, 10**6)
 
 
 def test_fails_health_check_for_large_non_base():
     @fails_health_check(HealthCheck.data_too_large)
     def _(data):
         if data.draw_integer(0, 2**8 - 1):
-            data.draw_bytes(10**6)
+            data.draw_bytes(10**6, 10**6)
 
 
 def test_fails_health_check_for_slow_draws():
@@ -487,10 +488,7 @@ def test_variable_size_string_increasing():
         return draw(st.text(st.characters(codec="ascii"), min_size=n, max_size=n))
 
     s = minimal(strategy(), lambda s: len(s) >= 5 and "a" in s)
-    # TODO_BETTER_SHRINK for the same reason as test_can_shrink_variable_string_draws.
-    # should be
-    # assert s == "0000a"
-    assert re.match("0+a", s)
+    assert s == "0000a"
 
 
 def test_run_nothing():
@@ -540,8 +538,8 @@ def test_can_write_bytes_towards_the_end():
 
     def f(data):
         if data.draw_boolean():
-            data.draw_bytes(5)
-            data.draw_bytes(len(buf), forced=buf)
+            data.draw_bytes(5, 5)
+            data.draw_bytes(len(buf), len(buf), forced=buf)
             assert data.buffer[-len(buf) :] == buf
 
     with buffer_size_limit(10):
@@ -552,7 +550,7 @@ def test_uniqueness_is_preserved_when_writing_at_beginning():
     seen = set()
 
     def f(data):
-        data.draw_bytes(1, forced=bytes(1))
+        data.draw_bytes(1, 1, forced=bytes(1))
         n = data.draw_integer(0, 2**3 - 1)
         assert n not in seen
         seen.add(n)
@@ -604,7 +602,7 @@ def test_detects_too_small_block_starts():
     def f(data):
         assert call_count[0] == 0
         call_count[0] += 1
-        data.draw_bytes(8)
+        data.draw_bytes(8, 8)
         data.mark_interesting()
 
     runner = ConjectureRunner(f, settings=settings(database=None))
@@ -955,7 +953,7 @@ def test_cached_test_function_does_not_reinvoke_on_prefix():
     def test_function(data):
         call_count[0] += 1
         data.draw_integer(0, 2**8 - 1)
-        data.draw_bytes(1, forced=bytes([7]))
+        data.draw_bytes(1, 1, forced=bytes([7]))
         data.draw_integer(0, 2**8 - 1)
 
     with deterministic_PRNG():
@@ -974,7 +972,7 @@ def test_will_evict_entries_from_the_cache(monkeypatch):
     count = [0]
 
     def tf(data):
-        data.draw_bytes(1)
+        data.draw_bytes(1, 1)
         count[0] += 1
 
     runner = ConjectureRunner(tf, settings=TEST_SETTINGS)
@@ -1424,7 +1422,7 @@ def test_does_not_run_optimisation_when_max_examples_is_small():
 
 def test_does_not_cache_extended_prefix():
     def test(data):
-        data.draw_bytes(8)
+        data.draw_bytes(8, 8)
 
     with deterministic_PRNG():
         runner = ConjectureRunner(test, settings=TEST_SETTINGS)
@@ -1441,7 +1439,7 @@ def test_does_cache_if_extend_is_not_used():
 
     def test(data):
         calls[0] += 1
-        data.draw_bytes(1)
+        data.draw_bytes(1, 1)
 
     with deterministic_PRNG():
         runner = ConjectureRunner(test, settings=TEST_SETTINGS)
@@ -1458,7 +1456,7 @@ def test_does_result_for_reuse():
 
     def test(data):
         calls[0] += 1
-        data.draw_bytes(1)
+        data.draw_bytes(1, 1)
 
     with deterministic_PRNG():
         runner = ConjectureRunner(test, settings=TEST_SETTINGS)
@@ -1472,7 +1470,7 @@ def test_does_result_for_reuse():
 
 def test_does_not_cache_overrun_if_extending():
     def test(data):
-        data.draw_bytes(8)
+        data.draw_bytes(8, 8)
 
     with deterministic_PRNG():
         runner = ConjectureRunner(test, settings=TEST_SETTINGS)
@@ -1485,8 +1483,8 @@ def test_does_not_cache_overrun_if_extending():
 
 def test_does_cache_overrun_if_not_extending():
     def test(data):
-        data.draw_bytes(8)
-        data.draw_bytes(8)
+        data.draw_bytes(8, 8)
+        data.draw_bytes(8, 8)
 
     with deterministic_PRNG():
         runner = ConjectureRunner(test, settings=TEST_SETTINGS)
@@ -1499,7 +1497,7 @@ def test_does_cache_overrun_if_not_extending():
 
 def test_does_not_cache_extended_prefix_if_overrun():
     def test(data):
-        data.draw_bytes(8)
+        data.draw_bytes(8, 8)
 
     with deterministic_PRNG():
         runner = ConjectureRunner(test, settings=TEST_SETTINGS)
@@ -1512,7 +1510,7 @@ def test_does_not_cache_extended_prefix_if_overrun():
 
 def test_can_be_set_to_ignore_limits():
     def test(data):
-        data.draw_bytes(1)
+        data.draw_bytes(1, 1)
 
     with deterministic_PRNG():
         runner = ConjectureRunner(
@@ -1556,71 +1554,6 @@ def _draw(cd, node):
     return getattr(cd, f"draw_{node.ir_type}")(**node.kwargs)
 
 
-@given(st.data())
-# drawing a second node with a different ir_type is hard to satisfy, as hypothesis
-# biases towards the first defined ir_type early on.
-@settings(suppress_health_check=[HealthCheck.too_slow, HealthCheck.filter_too_much])
-def test_extensions_of_misaligned_trees_are_cached(data):
-    # ConjectureData treats all ir node prefixes as if they were not forced,
-    # so was_forced doesn't make a difference when drawing. In fact, was_forced
-    # will always be False after running a tree through ConjectureData, so comparing
-    # for equality before and after is easiest if was_forced is always False.
-    node = data.draw(ir_nodes(was_forced=False))
-    misaligned_node = data.draw(ir_nodes(was_forced=False))
-    assume(node.ir_type != misaligned_node.ir_type)
-    # avoid trivial nodes resulting in exhausting the tree extremely early
-    assume(compute_max_children(node.ir_type, node.kwargs) > 100)
-
-    def test(cd):
-        _draw(cd, node)
-        _draw(cd, node)
-
-    runner = ConjectureRunner(test)
-
-    def _assert_cached(cd):
-        assert runner.call_count == 1
-        assert cd.status is Status.INVALID
-        assert cd.examples.ir_tree_nodes == [node]
-
-    assert runner.call_count == 0
-
-    cd = runner.cached_test_function_ir([node, misaligned_node])
-    _assert_cached(cd)
-
-    cd = runner.cached_test_function_ir([node, misaligned_node])
-    _assert_cached(cd)
-
-    extension = data.draw(st.lists(ir_nodes(was_forced=False)))
-    cd = runner.cached_test_function_ir([node, misaligned_node, *extension])
-    _assert_cached(cd)
-
-
-@given(ir_nodes(was_forced=False), ir_nodes(was_forced=False))
-@settings(suppress_health_check=[HealthCheck.too_slow, HealthCheck.filter_too_much])
-def test_misaligned_tree_does_not_clobber_cache(node, misaligned_node):
-    assume(node.ir_type != misaligned_node.ir_type)
-    assume(compute_max_children(node.ir_type, node.kwargs) > 100)
-
-    def test(cd):
-        _draw(cd, node)
-
-    runner = ConjectureRunner(test)
-    assert runner.call_count == 0
-
-    data_buffer = runner.cached_test_function(b"")
-    assert runner.call_count == 1
-    assert data_buffer.status is Status.OVERRUN
-
-    data_ir = runner.cached_test_function_ir([misaligned_node])
-    assert data_ir.status is Status.INVALID
-    assert data_ir.buffer == b""
-    assert runner.call_count == 2
-
-    result = runner.cached_test_function(b"")
-    assert runner.call_count == 2
-    assert result == data_buffer
-
-
 @pytest.mark.parametrize("forced_first", [True, False])
 @given(node=ir_nodes(was_forced=False))
 def test_cache_ignores_was_forced(forced_first, node):
