@@ -17,6 +17,7 @@ import pytest
 from hypothesis import HealthCheck, assume, example, given, settings, strategies as st
 from hypothesis.errors import StopTest
 from hypothesis.internal.conjecture.data import (
+    COLLECTION_DEFAULT_MAX_SIZE,
     ConjectureData,
     IRNode,
     Status,
@@ -35,9 +36,9 @@ from tests.common.debug import minimal
 from tests.conjecture.common import (
     draw_value,
     fresh_data,
+    integer_kwargs,
     ir_nodes,
     ir_types_and_kwargs,
-    kwargs_strategy,
 )
 
 
@@ -56,8 +57,7 @@ def test_compute_max_children_is_positive(ir_type_and_kwargs):
 @pytest.mark.parametrize(
     "ir_type, kwargs, count_children",
     [
-        ("integer", {"min_value": 1, "max_value": 2, "weights": [0, 1]}, 1),
-        ("integer", {"min_value": 1, "max_value": 4, "weights": [0, 0.5, 0, 0.5]}, 2),
+        ("integer", {"min_value": 1, "max_value": 2, "weights": {1: 0.1, 2: 0.1}}, 2),
         # only possibility is the empty string
         (
             "string",
@@ -88,7 +88,7 @@ def test_compute_max_children_is_positive(ir_type_and_kwargs):
             "string",
             {
                 "min_size": 0,
-                "max_size": None,
+                "max_size": COLLECTION_DEFAULT_MAX_SIZE,
                 "intervals": IntervalSet.from_string("a"),
             },
             MAX_CHILDREN_EFFECTIVELY_INFINITE,
@@ -102,6 +102,30 @@ def test_compute_max_children_is_positive(ir_type_and_kwargs):
             },
             MAX_CHILDREN_EFFECTIVELY_INFINITE,
         ),
+        (
+            "bytes",
+            {
+                "min_size": 0,
+                "max_size": 2,
+            },
+            sum(2 ** (8 * k) for k in range(2 + 1)),
+        ),
+        (
+            "bytes",
+            {
+                "min_size": 0,
+                "max_size": COLLECTION_DEFAULT_MAX_SIZE,
+            },
+            MAX_CHILDREN_EFFECTIVELY_INFINITE,
+        ),
+        (
+            "bytes",
+            {
+                "min_size": 0,
+                "max_size": 10_000,
+            },
+            MAX_CHILDREN_EFFECTIVELY_INFINITE,
+        ),
         ("boolean", {"p": 0.0}, 1),
         ("boolean", {"p": 1.0}, 1),
         ("boolean", {"p": 0.5}, 2),
@@ -250,11 +274,13 @@ def test_draw_string_single_interval_with_equal_bounds(s, n):
         {
             "min_value": 1,
             "max_value": 2,
-            "weights": [0, 1],
+            "weights": {1: 0.2, 2: 0.4},
+            "shrink_towards": 0,
         },
     )
 )
 @given(ir_types_and_kwargs())
+@settings(suppress_health_check=[HealthCheck.filter_too_much])
 def test_compute_max_children_and_all_children_agree(ir_type_and_kwargs):
     (ir_type, kwargs) = ir_type_and_kwargs
     max_children = compute_max_children(ir_type, kwargs)
@@ -272,13 +298,25 @@ def test_compute_max_children_and_all_children_agree(ir_type_and_kwargs):
 # compute_max_children, because they by necessity require iterating over 2**127
 # or more elements. We do the not great approximation of checking just the first
 # element is what we expect.
-@pytest.mark.parametrize(
-    "min_value, max_value, first",
-    [(None, None, -(2**127) + 1), (None, 42, (-(2**127) + 1) + 42), (42, None, 42)],
-)
-def test_compute_max_children_unbounded_integer_ranges(min_value, max_value, first):
-    kwargs = {"min_value": min_value, "max_value": max_value, "weights": None}
-    assert first == next(all_children("integer", kwargs))
+
+
+@given(integer_kwargs())
+def test_compute_max_children_integer_ranges(kwargs):
+    if kwargs["weights"] is not None:
+        # this case is in principle testable. would need to takewhile from all_children
+        # while weight is not zero.
+        assume(all(v > 0 for v in kwargs["weights"]))
+    if kwargs["min_value"] is not None:
+        expected = kwargs["min_value"]
+    else:
+        offset = (
+            0
+            if kwargs["max_value"] is None
+            else min(kwargs["max_value"], kwargs["shrink_towards"])
+        )
+        expected = offset - (2**127) + 1
+    first = next(all_children("integer", kwargs))
+    assert expected == first, (expected, first)
 
 
 @given(st.randoms())
@@ -289,7 +327,7 @@ def test_ir_nodes(random):
 
     data.start_example(42)
     data.draw_string(IntervalSet.from_string("abcd"), forced="abbcccdddd")
-    data.draw_bytes(8, forced=bytes(8))
+    data.draw_bytes(8, 8, forced=bytes(8))
     data.stop_example()
 
     data.draw_integer(0, 100, forced=50)
@@ -319,14 +357,14 @@ def test_ir_nodes(random):
             kwargs={
                 "intervals": IntervalSet.from_string("abcd"),
                 "min_size": 0,
-                "max_size": None,
+                "max_size": COLLECTION_DEFAULT_MAX_SIZE,
             },
             was_forced=True,
         ),
         IRNode(
             ir_type="bytes",
             value=bytes(8),
-            kwargs={"size": 8},
+            kwargs={"min_size": 8, "max_size": 8},
             was_forced=True,
         ),
         IRNode(
@@ -371,56 +409,6 @@ def test_data_with_empty_ir_tree_is_overrun():
     assert data.status is Status.OVERRUN
 
 
-@settings(suppress_health_check=[HealthCheck.too_slow])
-@given(st.data())
-def test_node_with_different_ir_type_is_invalid(data):
-    node = data.draw(ir_nodes())
-    (ir_type, kwargs) = data.draw(ir_types_and_kwargs())
-
-    # drawing a node with a different ir type should cause a misalignment.
-    assume(ir_type != node.ir_type)
-
-    data = ConjectureData.for_ir_tree([node])
-    draw_func = getattr(data, f"draw_{ir_type}")
-    with pytest.raises(StopTest):
-        draw_func(**kwargs)
-
-    assert data.status is Status.INVALID
-
-
-@settings(suppress_health_check=[HealthCheck.too_slow])
-@given(st.data())
-def test_node_with_same_ir_type_but_different_value_is_invalid(data):
-    node = data.draw(ir_nodes())
-    kwargs = data.draw(kwargs_strategy(node.ir_type))
-
-    # drawing a node with the same ir type, but a non-compatible value, should
-    # also cause a misalignment.
-    assume(not ir_value_permitted(node.value, node.ir_type, kwargs))
-
-    data = ConjectureData.for_ir_tree([node])
-    draw_func = getattr(data, f"draw_{node.ir_type}")
-    with pytest.raises(StopTest):
-        draw_func(**kwargs)
-
-    assert data.status is Status.INVALID
-
-
-@given(ir_nodes(was_forced=False))
-def test_data_with_changed_was_forced(node):
-    # we had a normal node and then tried to draw a different forced value from it.
-    # ir tree: v1 [was_forced=False]
-    # drawing:    [forced=v2]
-    data = ConjectureData.for_ir_tree([node])
-
-    draw_func = getattr(data, f"draw_{node.ir_type}")
-    kwargs = deepcopy(node.kwargs)
-    kwargs["forced"] = draw_value(node.ir_type, node.kwargs)
-    assume(not ir_value_equal(node.ir_type, kwargs["forced"], node.value))
-
-    assert ir_value_equal(node.ir_type, draw_func(**kwargs), kwargs["forced"])
-
-
 @given(ir_nodes(was_forced=True))
 def test_data_with_changed_forced_value(node):
     # we had a forced node and then tried to draw a different forced value from it.
@@ -482,7 +470,7 @@ def test_data_with_changed_forced_value(node):
         kwargs={
             "intervals": IntervalSet.from_string("bcda"),
             "min_size": 4,
-            "max_size": None,
+            "max_size": COLLECTION_DEFAULT_MAX_SIZE,
         },
         was_forced=True,
     )
@@ -491,7 +479,7 @@ def test_data_with_changed_forced_value(node):
     IRNode(
         ir_type="bytes",
         value=bytes(8),
-        kwargs={"size": 8},
+        kwargs={"min_size": 8, "max_size": 8},
         was_forced=True,
     )
 )
@@ -510,6 +498,7 @@ def test_data_with_same_forced_value_is_valid(node):
 
 
 @given(ir_types_and_kwargs())
+@settings(suppress_health_check=[HealthCheck.filter_too_much])
 def test_all_children_are_permitted_values(ir_type_and_kwargs):
     (ir_type, kwargs) = ir_type_and_kwargs
     max_children = compute_max_children(ir_type, kwargs)
@@ -617,8 +606,10 @@ def test_all_children_are_permitted_values(ir_type_and_kwargs):
             {"min_size": 1, "max_size": 10, "intervals": IntervalSet.from_string("e")},
             True,
         ),
-        (b"a", "bytes", {"size": 2}, False),
-        (b"aa", "bytes", {"size": 2}, True),
+        (b"a", "bytes", {"min_size": 2, "max_size": 2}, False),
+        (b"aa", "bytes", {"min_size": 2, "max_size": 2}, True),
+        (b"aa", "bytes", {"min_size": 0, "max_size": 3}, True),
+        (b"a", "bytes", {"min_size": 2, "max_size": 10}, False),
         (True, "boolean", {"p": 0}, False),
         (False, "boolean", {"p": 0}, True),
         (True, "boolean", {"p": 1}, True),
@@ -684,7 +675,7 @@ def test_forced_nodes_are_trivial(node):
             kwargs={
                 "intervals": IntervalSet.from_string("abcd"),
                 "min_size": 0,
-                "max_size": None,
+                "max_size": COLLECTION_DEFAULT_MAX_SIZE,
             },
             was_forced=False,
         ),
@@ -694,14 +685,20 @@ def test_forced_nodes_are_trivial(node):
             kwargs={
                 "intervals": IntervalSet.from_string("bcda"),
                 "min_size": 4,
-                "max_size": None,
+                "max_size": COLLECTION_DEFAULT_MAX_SIZE,
             },
             was_forced=False,
         ),
         IRNode(
             ir_type="bytes",
             value=bytes(8),
-            kwargs={"size": 8},
+            kwargs={"min_size": 8, "max_size": 8},
+            was_forced=False,
+        ),
+        IRNode(
+            ir_type="bytes",
+            value=bytes(2),
+            kwargs={"min_size": 2, "max_size": COLLECTION_DEFAULT_MAX_SIZE},
             was_forced=False,
         ),
         IRNode(
@@ -832,14 +829,26 @@ def test_trivial_nodes(node):
             kwargs={
                 "intervals": IntervalSet.from_string("abcd"),
                 "min_size": 1,
-                "max_size": None,
+                "max_size": COLLECTION_DEFAULT_MAX_SIZE,
             },
             was_forced=False,
         ),
         IRNode(
             ir_type="bytes",
             value=b"\x01",
-            kwargs={"size": 1},
+            kwargs={"min_size": 1, "max_size": 1},
+            was_forced=False,
+        ),
+        IRNode(
+            ir_type="bytes",
+            value=bytes(1),
+            kwargs={"min_size": 0, "max_size": COLLECTION_DEFAULT_MAX_SIZE},
+            was_forced=False,
+        ),
+        IRNode(
+            ir_type="bytes",
+            value=bytes(2),
+            kwargs={"min_size": 1, "max_size": 10},
             was_forced=False,
         ),
         IRNode(
