@@ -29,15 +29,15 @@ from hypothesis.strategies._internal.strategies import SearchStrategy
 @given(st.binary())
 def test_buffer_draws_as_self(buf):
     x = ConjectureData.for_buffer(buf)
-    assert x.draw_bytes(len(buf)) == buf
+    assert x.draw_bytes(len(buf), len(buf)) == buf
 
 
 def test_cannot_draw_after_freeze():
     x = ConjectureData.for_buffer(b"hi")
-    x.draw_bytes(1)
+    x.draw_bytes(1, 1)
     x.freeze()
     with pytest.raises(Frozen):
-        x.draw_bytes(1)
+        x.draw_bytes(1, 1)
 
 
 def test_can_double_freeze():
@@ -51,13 +51,13 @@ def test_can_double_freeze():
 def test_can_draw_zero_bytes():
     x = ConjectureData.for_buffer(b"")
     for _ in range(10):
-        assert x.draw_bytes(0) == b""
+        assert x.draw_bytes(0, 0) == b""
 
 
 def test_draw_past_end_sets_overflow():
     x = ConjectureData.for_buffer(bytes(5))
     with pytest.raises(StopTest) as e:
-        x.draw_bytes(6)
+        x.draw_bytes(6, 6)
     assert e.value.testcounter == x.testcounter
     assert x.frozen
     assert x.status == Status.OVERRUN
@@ -90,21 +90,6 @@ def test_can_mark_invalid():
     assert x.status == Status.INVALID
 
 
-@given(st.data(), st.integers(1, 100))
-def test_can_draw_weighted_integer_range(data, n):
-    weights = [1] * n + [0] * n
-    for _ in range(10):
-        # If the weights are working, then we'll never draw a value with weight=0
-        x = data.conjecture_data.draw_integer(1, 2 * n, weights=weights)
-        assert x <= n
-
-
-@given(st.binary(min_size=10))
-def test_can_draw_weighted_integer_range_2(buffer):
-    data = ConjectureData.for_buffer(buffer)
-    data.draw_integer(0, 7, weights=[1] * 8, shrink_towards=6)
-
-
 def test_can_mark_invalid_with_why():
     x = ConjectureData.for_buffer(b"")
     with pytest.raises(StopTest):
@@ -116,7 +101,7 @@ def test_can_mark_invalid_with_why():
 
 class BoomStrategy(SearchStrategy):
     def do_draw(self, data):
-        data.draw_bytes(1)
+        data.draw_bytes(1, 1)
         raise ValueError
 
 
@@ -130,7 +115,7 @@ def test_closes_interval_on_error_in_strategy():
 
 class BigStrategy(SearchStrategy):
     def do_draw(self, data):
-        data.draw_bytes(10**6)
+        data.draw_bytes(10**6, 10**6)
 
 
 def test_does_not_double_freeze_in_interval_close():
@@ -145,15 +130,18 @@ def test_triviality():
     d = ConjectureData.for_buffer([1, 0, 1])
 
     d.start_example(label=1)
-    d.draw_boolean()
-    d.draw_boolean()
+    d.draw(st.booleans())
+    d.draw(st.booleans())
+    d.stop_example()
+
+    d.start_example(label=2)
+    d.draw_bytes(1, 1, forced=bytes([2]))
     d.stop_example()
 
-    d.draw_bytes(1, forced=bytes([2]))
     d.freeze()
 
     def eg(u, v):
-        return next(ex for ex in d.examples if ex.start == u and ex.end == v)
+        return next(ex for ex in d.examples if ex.ir_start == u and ex.ir_end == v)
 
     assert not eg(0, 2).trivial
     assert not eg(0, 1).trivial
@@ -164,18 +152,29 @@ def test_triviality():
 def test_example_depth_marking():
     d = ConjectureData.for_buffer(bytes(24))
     # These draw sizes are chosen so that each example has a unique length.
-    d.draw_bytes(2)
+    d.draw(st.binary(min_size=2, max_size=2))
     d.start_example("inner")
-    d.draw_bytes(3)
-    d.draw_bytes(6)
+    d.draw(st.binary(min_size=3, max_size=3))
+    d.draw(st.binary(min_size=6, max_size=6))
     d.stop_example()
-    d.draw_bytes(12)
+    d.draw(st.binary(min_size=12, max_size=12))
     d.freeze()
 
-    assert len(d.examples) == 6
+    assert len(d.examples) == 10
 
     depths = {(ex.length, ex.depth) for ex in d.examples}
-    assert depths == {(2, 1), (3, 2), (6, 2), (9, 1), (12, 1), (23, 0)}
+    assert depths == {
+        (2, 1),
+        (2, 2),
+        (3, 2),
+        (3, 3),
+        (6, 2),
+        (6, 3),
+        (9, 1),
+        (12, 1),
+        (12, 2),
+        (23, 0),
+    }
 
 
 def test_has_examples_even_when_empty():
@@ -331,12 +330,20 @@ def test_examples_show_up_as_discarded():
 
 def test_examples_support_negative_indexing():
     d = ConjectureData.for_buffer(bytes(2))
-    d.draw_boolean()
-    d.draw_boolean()
+    d.draw(st.booleans())
+    d.draw(st.booleans())
     d.freeze()
     assert d.examples[-1].length == 1
 
 
+def test_examples_out_of_bounds_index():
+    d = ConjectureData.for_buffer(bytes(2))
+    d.draw(st.booleans())
+    d.freeze()
+    with pytest.raises(IndexError):
+        d.examples[10]
+
+
 def test_can_override_label():
     d = ConjectureData.for_buffer(bytes(2))
     d.draw(st.booleans(), label=7)
@@ -436,13 +443,13 @@ def test_child_indices():
     d = ConjectureData.for_buffer(bytes(4))
 
     d.start_example(0)  # examples[1]
-    d.start_example(0)  # examples[2]
-    d.draw_boolean()  # examples[3] + draw_bits (examples[4])
-    d.draw_boolean()  # examples[5] + draw_bits (examples[6])
+    d.start_example(1)  # examples[2]
+    d.draw(st.booleans())  # examples[3] (lazystrategy) + examples[4] (st.booleans)
+    d.draw(st.booleans())  # examples[4] (lazystrategy) + examples[5] (st.booleans)
     d.stop_example()
     d.stop_example()
-    d.draw_boolean()  # examples[7] + draw_bits (examples[8])
-    d.draw_boolean()  # examples[9] + draw_bits (examples[10])
+    d.draw(st.booleans())  # examples[7] (lazystrategy) + examples[8] (st.booleans)
+    d.draw(st.booleans())  # examples[9] (lazystrategy) + examples[10] (st.booleans)
     d.freeze()
 
     assert list(d.examples.children[0]) == [1, 7, 9]
@@ -482,7 +489,7 @@ def test_example_equality():
 def test_partial_buffer(n, rnd):
     data = ConjectureData(prefix=[n], random=rnd, max_length=2)
 
-    assert data.draw_bytes(2)[0] == n
+    assert data.draw_bytes(2, 2)[0] == n
 
 
 def test_structural_coverage_is_cached():
