@@ -8,6 +8,7 @@
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
 
+import datetime as dt
 import decimal
 import math
 import operator
@@ -18,8 +19,9 @@ from sys import float_info
 
 import pytest
 
-from hypothesis import given, settings, strategies as st
+from hypothesis import HealthCheck, given, settings, strategies as st
 from hypothesis.errors import HypothesisWarning, Unsatisfiable
+from hypothesis.internal.conjecture.data import COLLECTION_DEFAULT_MAX_SIZE
 from hypothesis.internal.filtering import max_len, min_len
 from hypothesis.internal.floats import next_down, next_up
 from hypothesis.internal.reflection import get_pretty_function_description
@@ -27,7 +29,7 @@ from hypothesis.strategies._internal.core import data
 from hypothesis.strategies._internal.lazy import LazyStrategy, unwrap_strategies
 from hypothesis.strategies._internal.numbers import FloatStrategy, IntegersStrategy
 from hypothesis.strategies._internal.strategies import FilteredStrategy, MappedStrategy
-from hypothesis.strategies._internal.strings import TextStrategy
+from hypothesis.strategies._internal.strings import BytesStrategy, TextStrategy
 
 from tests.common.debug import check_can_generate_examples
 from tests.common.utils import fails_with
@@ -262,6 +264,7 @@ Y = 2**20
         ]
     ),
 )
+@settings(suppress_health_check=[HealthCheck.too_slow])
 def test_rewrite_filter_chains_with_some_unhandled(data, predicates, s):
     # Set up our strategy
     for p in predicates:
@@ -504,6 +507,10 @@ def test_filter_rewriting_text_lambda_len(data, strategy, predicate, start, end)
     if isinstance(unwrapped.filtered_strategy, MappedStrategy):
         unwrapped = unwrapped.filtered_strategy.mapped_strategy
 
+    # binary() has a finite-but-effectively-infinite cap instead.
+    if isinstance(unwrapped_nofilter, BytesStrategy) and end == math.inf:
+        end = COLLECTION_DEFAULT_MAX_SIZE
+
     assert unwrapped.filtered_strategy.min_size == start
     assert unwrapped.filtered_strategy.max_size == end
     value = data.draw(s)
@@ -589,9 +596,11 @@ def test_does_not_rewrite_unsatisfiable_len_filter(predicate):
     "strategy, pattern",
     [
         (st.text(), "ab+c"),
+        (st.text(), "a|b"),
         (st.text(alphabet="abcdef"), "ab+c"),
         (st.text(min_size=5, max_size=10), "ab+c"),
         (st.binary(), b"ab+c"),
+        (st.binary(), b"a|b"),
         (st.binary(min_size=5, max_size=10), b"ab+c"),
     ],
     ids=repr,
@@ -616,3 +625,20 @@ def test_regex_filter_rewriting(data, strategy, pattern, method):
 @given(st.text().filter(re.compile("abc").sub))
 def test_error_on_method_which_requires_multiple_args(_):
     pass
+
+
+def test_dates_filter_rewriting():
+    today = dt.date.today()
+
+    assert st.dates().filter(partial(operator.lt, dt.date.max)).is_empty
+    assert st.dates().filter(partial(operator.gt, dt.date.min)).is_empty
+    assert st.dates(min_value=today).filter(partial(operator.gt, today)).is_empty
+    assert st.dates(max_value=today).filter(partial(operator.lt, today)).is_empty
+
+    bare = unwrap_strategies(st.dates())
+    assert bare.filter(partial(operator.ge, dt.date.max)) is bare
+    assert bare.filter(partial(operator.le, dt.date.min)) is bare
+
+    new = bare.filter(partial(operator.le, today))
+    assert not new.is_empty
+    assert new is not bare
