@@ -1,26 +1,23 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
-# (david@drmaciver.com), but it contains contributions by others. See
-# CONTRIBUTING.rst for a full list of people who may hold copyright, and
-# consult the git log if you need to determine who owns an individual
-# contribution.
+# Copyright the Hypothesis Authors.
+# Individual contributors are listed in AUTHORS.rst and the git log.
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
 # obtain one at https://mozilla.org/MPL/2.0/.
-#
-# END HEADER
 
 import io
-import random
+import unittest
 from operator import attrgetter
+from random import randbytes
 
 import pytest
 
 from hypothesis import Phase, given, settings, strategies as st
 from hypothesis.database import InMemoryExampleDatabase
+from hypothesis.errors import InvalidArgument
 from hypothesis.internal.conjecture.shrinker import sort_key
 
 
@@ -45,14 +42,15 @@ def test_fuzz_one_input(buffer_type):
 
     # Before running fuzz_one_input, there's nothing in `db`, and so the test passes
     # (because example generation is disabled by the custom settings)
-    test()
+    with pytest.raises(unittest.SkipTest):  # because this generates no examples
+        test()
     assert len(seen) == 0
 
     # If we run a lot of random bytestrings through fuzz_one_input, we'll eventually
     # find a failing example.
     with pytest.raises(AssertionError):
         for _ in range(1000):
-            buf = bytes(random.getrandbits(8) for _ in range(1000))
+            buf = randbytes(1000)
             seeds.append(buf)
             test.hypothesis.fuzz_one_input(buffer_type(buf))
 
@@ -63,7 +61,7 @@ def test_fuzz_one_input(buffer_type):
     # recent seed that we tried or the pruned-and-canonicalised form of it.
     (saved_examples,) = db.data.values()
     assert len(saved_examples) == 1
-    assert sort_key(seeds[-1]) >= sort_key(list(saved_examples)[0])
+    assert sort_key(seeds[-1]) >= sort_key(next(iter(saved_examples)))
 
     # Now that we have a failure in `db`, re-running our test is sufficient to
     # reproduce it, *and shrink to a minimal example*.
@@ -78,7 +76,7 @@ def test_can_fuzz_with_database_eq_None():
     @given(st.none())
     @settings(database=None)
     def test(s):
-        assert False
+        raise AssertionError
 
     with pytest.raises(AssertionError):
         test.hypothesis.fuzz_one_input(b"\x00\x00")
@@ -91,10 +89,10 @@ def test_fuzzing_unsatisfiable_test_always_returns_None():
     @given(st.none().filter(bool))
     @settings(database=None)
     def test(s):
-        assert False
+        raise AssertionError("Unreachable because there are no valid examples")
 
     for _ in range(100):
-        buf = bytes(random.getrandbits(8) for _ in range(3))
+        buf = randbytes(3)
         ret = test.hypothesis.fuzz_one_input(buf)
         assert ret is None
 
@@ -155,3 +153,14 @@ def test_fuzz_one_input_does_not_add_redundant_entries_to_database(buffers, db_s
     (saved_examples,) = db.data.values()
     assert seen == buffers
     assert len(saved_examples) == db_size
+
+
+def test_fuzzing_invalid_test_raises_error():
+    # Invalid: @given with too many positional arguments
+    @given(st.integers(), st.integers())
+    def invalid_test(s):
+        pass
+
+    with pytest.raises(InvalidArgument, match="Too many positional arguments"):
+        # access the property to check error happens during setup
+        invalid_test.hypothesis.fuzz_one_input
